@@ -546,7 +546,7 @@ lav_matrix_duplication_ginv <- .dup_ginv2
 # number of rows in A must be 'square' (n*n)
 lav_matrix_duplication_ginv_pre <- function(A = matrix(0,0,0)) {
 
-    A <- as.matrix(A)
+    A <- as.matrix.default(A)
 
     # number of rows
     n2 <- NROW(A)
@@ -567,7 +567,7 @@ lav_matrix_duplication_ginv_pre <- function(A = matrix(0,0,0)) {
 # number of columns in A must be 'square' (n*n)
 lav_matrix_duplication_ginv_post <- function(A = matrix(0,0,0)) {
 
-    A <- as.matrix(A)
+    A <- as.matrix.default(A)
 
     # number of columns
     n2 <- NCOL(A)
@@ -587,7 +587,7 @@ lav_matrix_duplication_ginv_post <- function(A = matrix(0,0,0)) {
 # for square matrices only, with ncol = nrow = n^2
 lav_matrix_duplication_ginv_pre_post <- function(A = matrix(0,0,0)) {
 
-    A <- as.matrix(A)
+    A <- as.matrix.default(A)
 
     # number of columns
     n2 <- NCOL(A)
@@ -1120,7 +1120,7 @@ lav_matrix_symmetric_inverse <- function(S, logdet   = FALSE,
             }
         }
     } else if(Sinv.method == "solve") {
-        S.inv <- solve(S)
+        S.inv <- solve.default(S)
         if(logdet) {
             ev <- eigen(S, symmetric = TRUE, only.values = TRUE)
             if(all(ev$values >= 0)) {
@@ -1131,7 +1131,7 @@ lav_matrix_symmetric_inverse <- function(S, logdet   = FALSE,
         }
     } else if(Sinv.method == "chol") {
         # this will break if S is not positive definite
-        cS <- chol(S)
+        cS <- chol.default(S)
         S.inv <- chol2inv(cS)
         if(logdet) {
             diag.cS <- diag(cS)
@@ -1156,7 +1156,7 @@ lav_matrix_symmetric_inverse <- function(S, logdet   = FALSE,
 # update inverse of A, after removing 1 or more rows (and corresponding
 # colums) from A
 #
-# - this is one of the many applications of the Sherman-Morrison formula
+# - this is just an application of the inverse of partitioned matrices
 # - only removal for now
 #
 lav_matrix_inverse_update <- function(A.inv, rm.idx = integer(0L)) {
@@ -1176,7 +1176,7 @@ lav_matrix_inverse_update <- function(A.inv, rm.idx = integer(0L)) {
         A <- A.inv[-rm.idx, rm.idx, drop = FALSE]
         B <- A.inv[ rm.idx,-rm.idx, drop = FALSE]
         H <- A.inv[ rm.idx, rm.idx, drop = FALSE]
-        out <- A.inv[-rm.idx, -rm.idx, drop = FALSE] - A %*% solve(H, B)
+        out <- A.inv[-rm.idx, -rm.idx, drop = FALSE] - A %*% solve.default(H, B)
 
     # erase all col/rows...
     } else if(ndel == NCOL(A.inv)) {
@@ -1218,10 +1218,12 @@ lav_matrix_symmetric_inverse_update <- function(S.inv, rm.idx = integer(0L),
     else if(ndel < NCOL(S.inv)) {
         A <- S.inv[ rm.idx, -rm.idx, drop = FALSE]
         H <- S.inv[ rm.idx,  rm.idx, drop = FALSE]
-        out <- S.inv[-rm.idx, -rm.idx, drop = FALSE] - crossprod(A, solve(H, A))
+        out <- ( S.inv[-rm.idx, -rm.idx, drop = FALSE] -
+                 crossprod(A, solve.default(H, A)) )
         if(logdet) {
-            cH <- chol(H); diag.cH <- diag(cH)
-            H.logdet <- sum(log(diag.cH * diag.cH))
+            #cH <- chol.default(Re(H)); diag.cH <- diag(cH)
+            #H.logdet <- sum(log(diag.cH * diag.cH))
+            H.logdet <- log(det(H))
             attr(out, "logdet") <- S.logdet + H.logdet
         }
 
@@ -1281,7 +1283,7 @@ lav_matrix_symmetric_det_update <- function(det.S, S.inv, rm.idx = integer(0L)){
     # rank-n update
     else if(ndel < NCOL(S.inv)) {
         H <- S.inv[ rm.idx, rm.idx, drop = FALSE]
-        cH <- chol(H); diag.cH <- diag(cH)
+        cH <- chol.default(H); diag.cH <- diag(cH)
         det.H <- prod(diag.cH * diag.cH)
         out <- det.S * det.H
 
@@ -1310,7 +1312,7 @@ lav_matrix_symmetric_logdet_update <- function(S.logdet, S.inv,
     # rank-n update
     else if(ndel < NCOL(S.inv)) {
         H <- S.inv[ rm.idx, rm.idx, drop = FALSE]
-        cH <- chol(H); diag.cH <- diag(cH)
+        cH <- chol.default(H); diag.cH <- diag(cH)
         H.logdet <- sum(log(diag.cH * diag.cH))
         out <- S.logdet + H.logdet
 
@@ -1322,6 +1324,98 @@ lav_matrix_symmetric_logdet_update <- function(S.logdet, S.inv,
     out
 }
 
+# compute `lambda': the smallest root of the determinantal equation
+# |M - lambda*P| = 0 (see Fuller 1987, p.125 or p.172
+#
+# the function allows for zero rows/columns in P, by regressing them out
+# this approach was suggested to me by Wayne A. Fuller, personal communication,
+# 12 Nov 2020
+#
+lav_matrix_symmetric_diff_smallest_root <- function(M = NULL, P = NULL,
+                                                    warn = FALSE) {
+
+    # check input (we will 'assume' they are square and symmetric)
+    stopifnot(is.matrix(M), is.matrix(P))
+
+    # check if P is diagonal or not
+    PdiagFlag <- FALSE
+    tmp <- P
+    diag(tmp) <- 0
+    if( all(abs(tmp) < sqrt(.Machine$double.eps)) ) {
+        PdiagFlag <- TRUE
+    }
+
+    # diagonal elements of P
+    nP <- nrow(P)
+    diagP <- P[lav_matrix_diag_idx(nP)]
+
+    # force diagonal elements of P to be nonnegative (warn?)
+    neg.idx <- which(diagP < 0)
+    if(length(neg.idx) > 0L) {
+        if(warn) {
+            warning("some diagonal elements of P are negative (and set to zero)")
+        }
+        diag(P)[neg.idx] <- diagP[neg.idx] <- 0
+    }
+
+    # check for (near)zero diagonal elements
+    zero.idx <- which(abs(diagP) < sqrt(.Machine$double.eps))
+
+    # three cases:
+    #    1. all elements are zero (P=0) -> lambda = 0
+    #    2. no elements are zero
+    #    3. some elements are zero -> regress out
+
+    # 1. all elements are zero
+    if(length(zero.idx) == nP) {
+        return(0.0)
+    }
+
+    # 2. no elements are zero
+    else if(length(zero.idx) == 0L) {
+        if(PdiagFlag) {
+            Ldiag <- 1/sqrt(diagP)
+            LML <- t(Ldiag * M) * Ldiag
+        } else {
+            L <- solve(lav_matrix_symmetric_sqrt(P))
+            LML <- L %*% M %*% t(L)
+        }
+
+        # compute lambda
+        lambda <- eigen(LML, symmetric = TRUE, only.values = TRUE)$values[nP]
+
+    # 3. some elements are zero
+    } else {
+        # regress out M-block corresponding to zero diagonal elements in P
+
+        # partition M accordingly: p = positive, n = negative
+        M.pp <- M[-zero.idx, -zero.idx, drop = FALSE]
+        M.pn <- M[-zero.idx,  zero.idx, drop = FALSE]
+        M.np <- M[ zero.idx, -zero.idx, drop = FALSE]
+        M.nn <- M[ zero.idx,  zero.idx, drop = FALSE]
+
+        # create Mp.n
+        Mp.n <- M.pp - M.pn %*% solve(M.nn) %*% M.np
+
+        # extract positive part of P
+        P.p <- P[-zero.idx, -zero.idx, drop = FALSE]
+
+        # compute smallest root
+        if(PdiagFlag) {
+            diagPp <- diag(P.p)
+            Ldiag <- 1/sqrt(diagPp)
+            LML <- t(Ldiag * Mp.n) * Ldiag
+        } else {
+            L <- solve(lav_matrix_symmetric_sqrt(P.p))
+            LML <- L %*% Mp.n %*% t(L)
+        }
+        lambda <- eigen(LML, symmetric = TRUE,
+                        only.values = TRUE)$values[nrow(P.p)]
+    }
+
+    lambda
+}
+
 # force a symmetric matrix to be positive definite
 # simple textbook version (see Matrix::nearPD for a more sophisticated version)
 #
@@ -1371,13 +1465,13 @@ lav_matrix_transform_mean_cov <- function(Y,
                                           target.mean = numeric( NCOL(Y) ),
                                           target.cov = diag( NCOL(Y) )) {
     # coerce to matrix
-    Y <- as.matrix(Y)
+    Y <- as.matrix.default(Y)
 
     # convert to vector
     target.mean <- as.vector(target.mean)
 
     S <- lav_matrix_cov(Y)
-    S.inv <- solve(S)
+    S.inv <- solve.default(S)
     S.inv.sqrt <- lav_matrix_symmetric_sqrt(S.inv)
     target.cov.sqrt <- lav_matrix_symmetric_sqrt(target.cov)
 
@@ -1400,7 +1494,7 @@ lav_matrix_transform_mean_cov <- function(Y,
 #
 lav_matrix_mean_wt <- function(Y, wt = NULL) {
 
-    Y <- unname(as.matrix(Y))
+    Y <- unname(as.matrix.default(Y))
     DIM <- dim(Y)
 
     if(is.null(wt)) {
@@ -1433,7 +1527,7 @@ lav_matrix_mean_wt <- function(Y, wt = NULL) {
 #
 lav_matrix_var_wt <- function(Y, wt = NULL, method = c("unbiased", "ML")) {
 
-    Y <- unname(as.matrix(Y))
+    Y <- unname(as.matrix.default(Y))
     DIM <- dim(Y)
 
     if(is.null(wt)) {
@@ -1470,7 +1564,7 @@ lav_matrix_var_wt <- function(Y, wt = NULL, method = c("unbiased", "ML")) {
 #
 lav_matrix_cov_wt <- function(Y, wt = NULL) {
 
-    Y <- unname(as.matrix(Y))
+    Y <- unname(as.matrix.default(Y))
     DIM <- dim(Y)
 
     if(is.null(wt)) {
@@ -1496,5 +1590,71 @@ lav_matrix_cov_wt <- function(Y, wt = NULL) {
     out
 }
 
+# compute (I-A)^{-1} where A is square
+# using a (truncated) Neumann series:  (I-A)^{-1} = \sum_k=0^{\infty} A^k
+#
+# as A is typically sparse, we can stop if all elements in A^k are zero for,
+# say, k<=6
+lav_matrix_inverse_iminus <- function(A = NULL) {
 
+    nr <- nrow(A); nc <- ncol(A)
+    stopifnot(nr == nc)
+
+    # create I + A
+    IA <- A
+    diag.idx <- lav_matrix_diag_idx(nr)
+    IA[diag.idx] <- IA[diag.idx] + 1
+
+    # initial approximation
+    IA.inv <- IA
+
+    # first order
+    A2 <- A %*% A
+    if(all(A2 == 0)) {
+        # we are done
+        return(IA.inv)
+    } else {
+        IA.inv <- IA.inv + A2
+    }
+
+    # second order
+    A3 <- A2 %*% A
+	if(all(A3 == 0)) {
+        # we are done
+        return(IA.inv)
+    } else {
+        IA.inv <- IA.inv + A3
+    }
+
+    # third order
+    A4 <- A3 %*% A
+    if(all(A4 == 0)) {
+        # we are done
+        return(IA.inv)
+    } else {
+        IA.inv <- IA.inv + A4
+    }
+
+    # fourth order
+    A5 <- A4 %*% A
+    if(all(A5 == 0)) {
+        # we are done
+        return(IA.inv)
+    } else {
+        IA.inv <- IA.inv + A5
+    }
+
+    # fifth order
+    A6 <- A5 %*% A
+    if(all(A6 == 0)) {
+        # we are done
+        return(IA.inv)
+    } else {
+        # naive version (for now)
+        tmp <- -A
+        tmp[diag.idx] <- tmp[diag.idx] + 1
+        IA.inv <- solve(tmp)
+        return(IA.inv)
+    }
+}
 
