@@ -23,6 +23,11 @@ lav_model_gradient <- function(lavmodel       = NULL,
     num.idx        <- lavmodel@num.idx
     th.idx         <- lavmodel@th.idx
     nx.free        <- lavmodel@nx.free
+    if(.hasSlot(lavmodel, "estimator.args")) {
+        estimator.args <- lavmodel@estimator.args
+    } else {
+        estimator.args <- list()
+    }
 
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
@@ -34,6 +39,12 @@ lav_model_gradient <- function(lavmodel       = NULL,
     if(group.weight) {
         if(estimator %in% c("ML","PML","FML","MML","REML","NTRLS")) {
             group.w <- (unlist(lavsamplestats@nobs)/lavsamplestats@ntotal)
+        } else if(estimator == "DLS") {
+            if(estimator.args$dls.FtimesNminus1) {
+                group.w <- ((unlist(lavsamplestats@nobs)-1)/lavsamplestats@ntotal)
+            } else {
+                group.w <- (unlist(lavsamplestats@nobs)/lavsamplestats@ntotal)
+            }
         } else {
             # FIXME: double check!
             group.w <- ((unlist(lavsamplestats@nobs)-1)/lavsamplestats@ntotal)
@@ -43,7 +54,7 @@ lav_model_gradient <- function(lavmodel       = NULL,
     }
 
     # do we need WLS.est?
-    if(estimator %in% c("WLS", "DWLS", "ULS", "GLS", "NTRLS")) {
+    if(estimator %in% c("WLS", "DWLS", "ULS", "GLS", "NTRLS", "DLS")) {
 
         # always compute WLS.est
         WLS.est <- lav_model_wls_est(lavmodel = lavmodel, GLIST = GLIST) #,
@@ -61,7 +72,6 @@ lav_model_gradient <- function(lavmodel       = NULL,
                              extra = (estimator %in% c("ML", "REML", "NTRLS")))
         #}
 
-        # ridge here?
         if(meanstructure) {
             #if(conditional.x) {
             #    Mu.hat <- computeMuHat(lavmodel = lavmodel, GLIST = GLIST)
@@ -83,6 +93,10 @@ lav_model_gradient <- function(lavmodel       = NULL,
         if(group.w.free) {
             GW <- computeGW(lavmodel = lavmodel, GLIST = GLIST)
         }
+    } else if(estimator == "DLS" && estimator.args$dls.GammaNT == "model") {
+        Sigma.hat <- computeSigmaHat(lavmodel = lavmodel, GLIST = GLIST,
+                                         extra = FALSE)
+        Mu.hat <- computeMuHat(lavmodel = lavmodel, GLIST = GLIST)
     } else if(estimator == "MML") {
         TH    <- computeTH(   lavmodel = lavmodel, GLIST = GLIST)
         THETA <- computeTHETA(lavmodel = lavmodel, GLIST = GLIST)
@@ -139,8 +153,14 @@ lav_model_gradient <- function(lavmodel       = NULL,
 
                 # only save what we need
                 DX[mm.in.group] <- DX.group[ mm.names ]
+            } else if(representation == "RAM") {
+                DX.group <- lav_ram_df(GLIST[mm.in.group],
+                                       Omega[[g]],
+                                       Omega.mu[[g]])
+                # only save what we need
+                DX[mm.in.group] <- DX.group[ mm.names ]
             } else {
-                stop("only representation LISREL has been implemented for now")
+                stop("only LISREL and RAM representation has been implemented for now")
             }
 
             # weight by group
@@ -176,7 +196,7 @@ lav_model_gradient <- function(lavmodel       = NULL,
     } else # ML
 
     # 2. using Delta - *LS family
-    if(estimator %in% c("WLS", "DWLS", "ULS", "GLS", "NTGLS")) {
+    if(estimator %in% c("WLS", "DWLS", "ULS", "GLS", "NTGLS", "DLS")) {
 
         if(type != "free") {
             if(is.null(Delta))
@@ -192,6 +212,7 @@ lav_model_gradient <- function(lavmodel       = NULL,
             # 0.5-17: use crossprod twice; treat DWLS/ULS special
             if(estimator == "WLS" ||
                estimator == "GLS" ||
+               estimator == "DLS" ||
                estimator == "NTRLS") {
                 # full weight matrix
                 diff <- lavsamplestats@WLS.obs[[g]]  - WLS.est[[g]]
@@ -201,6 +222,25 @@ lav_model_gradient <- function(lavmodel       = NULL,
                     WLS.V <- lavsamplestats@WLS.V[[g]]
                     group.dx <- -1 * crossprod(Delta[[g]],
                                                crossprod(WLS.V, diff))
+                } else if(estimator == "DLS") {
+                    if(estimator.args$dls.GammaNT == "sample") {
+                        WLS.V <- lavsamplestats@WLS.V[[g]] # for now
+                    } else {
+                        dls.a <- estimator.args$dls.a
+                        GammaNT <- lav_samplestats_Gamma_NT(
+                            COV            = Sigma.hat[[g]],
+                            MEAN           = Mu.hat[[g]],
+                            rescale        = FALSE,
+                            x.idx          = lavsamplestats@x.idx[[g]],
+                            fixed.x        = lavmodel@fixed.x,
+                            conditional.x  = lavmodel@conditional.x,
+                            meanstructure  = lavmodel@meanstructure,
+                            slopestructure = lavmodel@conditional.x)
+                        W.DLS <- (1 - dls.a)*lavsamplestats@NACOV[[g]] + dls.a*GammaNT
+                        WLS.V <- lav_matrix_symmetric_inverse(W.DLS)
+                    }
+                    group.dx <- -1 * crossprod(Delta[[g]],
+                                               crossprod(WLS.V, diff))
                 } else if(estimator == "NTRLS") {
                     stopifnot(!conditional.x)
                     #WLS.V <- lav_samplestats_Gamma_inverse_NT(
@@ -365,14 +405,28 @@ lav_model_gradient <- function(lavmodel       = NULL,
 
         # for each upper-level group....
         for(g in 1:lavmodel@ngroups) {
-            DX <- lav_mvnorm_cluster_dlogl_2l_samplestats(
-                       YLp = lavsamplestats@YLp[[g]],
-                       Lp  = lavdata@Lp[[g]],
-                       Mu.W    = Mu.hat[[(g-1)*2 + 1]],
-                       Sigma.W = Sigma.hat[[(g-1)*2 + 1]],
-                       Mu.B    = Mu.hat[[(g-1)*2 + 2]],
-                       Sigma.B = Sigma.hat[[(g-1)*2 + 2]],
-                       Sinv.method  = "eigen")
+            if(!lavsamplestats@missing.flag) { # complete data
+                DX <- lav_mvnorm_cluster_dlogl_2l_samplestats(
+                           YLp = lavsamplestats@YLp[[g]],
+                           Lp  = lavdata@Lp[[g]],
+                           Mu.W    = Mu.hat[[(g-1)*2 + 1]],
+                           Sigma.W = Sigma.hat[[(g-1)*2 + 1]],
+                           Mu.B    = Mu.hat[[(g-1)*2 + 2]],
+                           Sigma.B = Sigma.hat[[(g-1)*2 + 2]],
+                           Sinv.method  = "eigen")
+            } else {
+                # missing data
+                DX <- lav_mvnorm_cluster_missing_dlogl_2l_samplestats(
+                           Y1 = lavdata@X[[g]],
+                           Y2 = lavsamplestats@YLp[[g]][[2]]$Y2,
+                           Lp  = lavdata@Lp[[g]],
+                           Mp = lavdata@Mp[[g]],
+                           Mu.W    = Mu.hat[[(g-1)*2 + 1]],
+                           Sigma.W = Sigma.hat[[(g-1)*2 + 1]],
+                           Mu.B    = Mu.hat[[(g-1)*2 + 2]],
+                           Sigma.B = Sigma.hat[[(g-1)*2 + 2]],
+                           Sinv.method  = "eigen")
+            }
 
             group.dx <- as.numeric( DX %*% Delta[[g]] )
 
@@ -654,7 +708,7 @@ computeDelta <- function(lavmodel = NULL, GLIST. = NULL,
         #}
 
         # if theta, do some preparation
-        if(parameterization == "theta") {
+        if(representation == "LISREL" && parameterization == "theta") {
             sigma.hat <- computeSigmaHat.LISREL(MLIST=GLIST[mm.in.group],
                                                 delta=FALSE)
             dsigma <- diag(sigma.hat)
@@ -777,8 +831,19 @@ computeDelta <- function(lavmodel = NULL, GLIST. = NULL,
                                                      MLIST=GLIST[ mm.in.group ])
                     DELTA <- rbind(DELTA.gw, DELTA)
                 }
+            } else if(representation == "RAM") {
+                DELTA <- dxSigma <-
+                    lav_ram_dsigma(m     = mname,
+                                   idx   = m.el.idx[[mm]],
+                                   MLIST = GLIST[ mm.in.group ])
+                if(lavmodel@meanstructure) {
+                    DELTA.mu <- lav_ram_dmu(m    = mname,
+                                           idx   = m.el.idx[[mm]],
+                                           MLIST = GLIST[ mm.in.group ])
+                    DELTA <- rbind(DELTA.mu, DELTA)
+                }
             } else {
-                stop("representation", representation, "not implemented yet")
+                stop("representation ", representation, " not implemented yet")
             }
 
             Delta.group[ ,x.el.idx[[mm]]] <- DELTA
