@@ -24,7 +24,8 @@ lav_partable_flat <- function(FLAT = NULL,
                               varTable         = NULL,
                               group.equal      = NULL,
                               group.w.free     = FALSE,
-                              ngroups          = 1L) {
+                              ngroups          = 1L,
+                              ov.names.x.block = NULL) {
 
     categorical <- FALSE
 
@@ -38,6 +39,9 @@ lav_partable_flat <- function(FLAT = NULL,
     lv.names.f   <- lav_partable_vnames(FLAT, type="lv.formative") # formative latent variables
     ov.names     <- lav_partable_vnames(FLAT, type="ov")     # observed variables
     ov.names.x   <- lav_partable_vnames(FLAT, type="ov.x")   # exogenous x covariates
+    if(is.null(ov.names.x.block)) {
+        ov.names.x.block <- ov.names.x
+    }
     ov.names.nox <- lav_partable_vnames(FLAT, type="ov.nox")
     lv.names.x   <- lav_partable_vnames(FLAT, type="lv.x")   # exogenous lv
     ov.names.y   <- lav_partable_vnames(FLAT, type="ov.y")   # dependent ov
@@ -307,7 +311,7 @@ lav_partable_flat <- function(FLAT = NULL,
                          user == 0L)
         ustart[var.idx] <- 0.0
           free[var.idx] <- 0L
-    } else {
+    } else if(length(lv.names.f) > 0L) {
         # 'formative' (residual) variances are set to zero by default
         var.idx <- which(op == "~~" &
                          lhs == rhs &
@@ -326,7 +330,7 @@ lav_partable_flat <- function(FLAT = NULL,
         ustart[lv.var.idx] <- 1.0
           free[lv.var.idx] <- 0L
     }
-    if(auto.efa) {
+    if(auto.efa && length(lv.names.efa) > 0L) {
         # fix lv variances of efa blocks to unity
         lv.var.idx <- which(op == "~~" &
                             lhs %in% lv.names.efa & lhs == rhs)
@@ -352,10 +356,11 @@ lav_partable_flat <- function(FLAT = NULL,
             # get corresponding indicator if unique
             lhs.mm <- lhs[mm.idx]; rhs.mm <- rhs[mm.idx]
             single.ind <- rhs.mm[which(lhs.mm %in% lv.names.single &
+                                       lhs.mm != rhs.mm & # exclude phantom
                                        !(duplicated(rhs.mm) |
                                          duplicated(rhs.mm, fromLast=TRUE)))]
             # is the indicator unique?
-            if(length(single.ind)) {
+            if(length(single.ind) > 0L) {
                 var.idx <- which(op == "~~" & lhs %in% single.ind
                                             & rhs %in% single.ind
                                             & lhs == rhs
@@ -407,7 +412,6 @@ lav_partable_flat <- function(FLAT = NULL,
           free[lv.cov.idx] <- 0L
     }
 
-
     # 4. intercepts
     if(meanstructure) {
         if(categorical) {
@@ -434,14 +438,30 @@ lav_partable_flat <- function(FLAT = NULL,
             ustart[lv.int.idx] <- 0.0
               free[lv.int.idx] <- 0L
         }
+        # 4b. fixed effect (only if we have random slopes)
+        if(!is.null(FLAT$rv) && any(nchar(FLAT$rv) > 0L)) {
+            lv.names.rv <- lav_partable_vnames(FLAT, "lv.rv")
+            lv.rv.idx <- which(op == "~1" &
+                               lhs %in% lv.names.rv &
+                               user == 0L)
+            ustart[lv.rv.idx] <- as.numeric(NA)
+              free[lv.rv.idx] <- 1L
+        }
     }
 
+    # 4b. fixed effect (only if we have random slopes)
+    #if(!is.null(FLAT$rv)) {
+    #    }
+
     # 5. handle exogenous `x' covariates
-    if(length(ov.names.x) > 0) {
+    # usually, ov.names.x.block == ov.names.x
+    # except if multilevel, where 'splitted' ov.x are treated as endogenous
+    if(length(ov.names.x.block) > 0) {
 
         # 1. variances/covariances
         exo.var.idx  <- which(op == "~~" &
-                          rhs %in% ov.names.x &
+                          rhs %in% ov.names.x.block &
+                          lhs %in% ov.names.x.block &
                           user == 0L)
         if(fixed.x) {
             ustart[exo.var.idx] <- as.numeric(NA) # should be overriden later!
@@ -453,7 +473,7 @@ lav_partable_flat <- function(FLAT = NULL,
 
         # 2. intercepts
         exo.int.idx  <- which(op == "~1" &
-                              lhs %in% ov.names.x &
+                              lhs %in% ov.names.x.block &
                               user == 0L)
         if(fixed.x) {
             ustart[exo.int.idx] <- as.numeric(NA) # should be overriden later!
@@ -464,9 +484,9 @@ lav_partable_flat <- function(FLAT = NULL,
         #}
 
         # 3. regressions ov + lv
-        exo.reg.idx <- which(op == "~" &
+        exo.reg.idx <- which(op %in% c("~", "<~") &
                              lhs %in% c(lv.names, ov.names.nox) &
-                             rhs %in% ov.names.x)
+                             rhs %in% ov.names.x.block)
         if(conditional.x) {
             exo[exo.reg.idx] <- 1L
         }
