@@ -1,5 +1,7 @@
 # YR - 26 Nov 2013: generate partable for the unrestricted model
 # YR - 19 Mar 2017: handle twolevel model
+# YR - 27 May 2021: added lav_partable_unrestricted_chol so we can use
+#                   a cholesky parameterization: S = LAMBDA %*% t(LAMBDA)
 
 lav_partable_unrestricted <- function(lavobject      = NULL,
                                       # if no object is available,
@@ -7,6 +9,7 @@ lav_partable_unrestricted <- function(lavobject      = NULL,
                                       lavpta         = NULL,
                                       lavoptions     = NULL,
                                       lavsamplestats = NULL,
+                                      lavh1          = NULL,
                                       # optional user-provided sample stats
                                       sample.cov     = NULL,
                                       sample.mean    = NULL,
@@ -18,8 +21,9 @@ lav_partable_unrestricted <- function(lavobject      = NULL,
 
     lav_partable_indep_or_unrestricted(lavobject = lavobject,
         lavdata = lavdata, lavpta = lavpta, lavoptions = lavoptions,
-        lavsamplestats = lavsamplestats, sample.cov = sample.cov,
-        sample.mean = sample.mean , sample.slopes = sample.slopes,
+        lavsamplestats = lavsamplestats, lavh1 = lavh1,
+        sample.cov = sample.cov, sample.mean = sample.mean ,
+        sample.slopes = sample.slopes,
         sample.th = sample.th, sample.th.idx = sample.th.idx,
         independent = FALSE)
 
@@ -33,6 +37,7 @@ lav_partable_independence <- function(lavobject      = NULL,
                                       lavpta         = NULL,
                                       lavoptions     = NULL,
                                       lavsamplestats = NULL,
+                                      lavh1          = NULL,
                                       # optional user-provided sample stats
                                       sample.cov     = NULL,
                                       sample.mean    = NULL,
@@ -44,8 +49,9 @@ lav_partable_independence <- function(lavobject      = NULL,
 
     lav_partable_indep_or_unrestricted(lavobject = lavobject,
         lavdata = lavdata, lavpta = lavpta, lavoptions = lavoptions,
-        lavsamplestats = lavsamplestats, sample.cov = sample.cov,
-        sample.mean = sample.mean , sample.slopes = sample.slopes,
+        lavsamplestats = lavsamplestats, lavh1 = lavh1,
+        sample.cov = sample.cov, sample.mean = sample.mean ,
+        sample.slopes = sample.slopes,
         sample.th = sample.th, sample.th.idx = sample.th.idx,
         independent = TRUE)
 }
@@ -56,6 +62,7 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
                                       lavpta         = NULL,
                                       lavoptions     = NULL,
                                       lavsamplestats = NULL,
+                                      lavh1          = NULL,
                                       # optional user-provided sample stats
                                       sample.cov     = NULL,
                                       sample.mean    = NULL,
@@ -74,6 +81,7 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
         lavoptions <- lavobject@Options
         lavsamplestats <- lavobject@SampleStats
         lavpta <- lavobject@pta
+        lavh1 <- lavobject@h1
     }
 
     # conditional.x ? check res.cov[[1]] slot
@@ -84,6 +92,11 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
         conditional.x <- TRUE
     }
 
+    # group.w.free?
+    group.w.free <- FALSE
+    if(!is.null(lavoptions) && lavoptions$group.w.free) {
+        group.w.free <- TRUE
+    }
     # we use CAPS below for the list version, so we can use 'small caps'
     # within the for() loop
 
@@ -166,7 +179,6 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
 
         # only for multilevel
         if(nlevels > 1L) {
-            OV.NAMES <- lavdata@ov.names[[g]]
             YLp <-  lavsamplestats@YLp[[g]]
             Lp <- lavdata@Lp[[g]]
         }
@@ -181,7 +193,7 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
         sample.mean.x <- SAMPLE.mean.x[[g]]
 
         # force local sample.cov to be pd -- just for starting values anyway
-        if(!is.null(sample.cov)) {
+        if(!is.null(sample.cov) && !anyNA(sample.cov)) {
             sample.cov <- lav_matrix_symmetric_force_pd(sample.cov)
         }
 
@@ -209,27 +221,37 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
             if(nlevels > 1L) {
 
                 if(independent) {
-                    ov.names.x <- lavpta$vnames$ov.x[[b]]
+                    # beter use lavdata@Lp[[g]]$ov.x.idx??
+                    # in case we have x/y mismatch across levels?
+                    ov.x.idx <- lavpta$vidx$ov.x[[b]]
+                    ov.names.x <-   lavpta$vnames$ov.x[[b]]
                     ov.names.nox <- lavpta$vnames$ov.nox[[b]]
-                    sample.cov.x <- NULL # we will estimate them
-                    sample.mean.x <- NULL
+                    sample.cov.x <- lavh1$implied$cov[[b]][ov.x.idx,
+                                                      ov.x.idx, drop = FALSE]
+                    sample.mean.x <- lavh1$implied$mean[[b]][ov.x.idx]
                 } else {
                     ov.names.x <- character(0L)
                     ov.names.nox <- ov.names
                 }
 
-                block.idx <- match(ov.names, OV.NAMES)
-
-                if(l == 1L) {
-                    sample.cov  <- YLp[[2]]$Sigma.W[block.idx, block.idx,
-                                                    drop = FALSE]
-                    sample.mean <- YLp[[2]]$Mu.W[block.idx]
+                if(length(lavh1) > 0L) {
+                    sample.cov  <- lavh1$implied$cov[[b]]
+                    sample.mean <- lavh1$implied$mean[[b]]
                 } else {
-                    sample.cov  <- YLp[[2]]$Sigma.B[block.idx, block.idx,
-                                                    drop = FALSE]
-                    sample.mean <- YLp[[2]]$Mu.B[block.idx]
+                    sample.cov <- diag(length(ov.names))
+                    sample.mean <- numeric(length(ov.names))
                 }
 
+                #if(l == 1L) {
+                #    sample.cov  <- YLp[[2]]$Sigma.W[block.idx, block.idx,
+                #                                    drop = FALSE]
+                #    sample.mean <- YLp[[2]]$Mu.W[block.idx]
+                #} else {
+                #    sample.cov  <- YLp[[2]]$Sigma.B[block.idx, block.idx,
+                #                                    drop = FALSE]
+                #    sample.mean <- YLp[[2]]$Mu.B[block.idx]
+                #}
+
                 # force local sample.cov to be strictly pd (and exaggerate)
                 # just for starting values anyway, but at least the first
                 # evaluation will be feasible
@@ -555,7 +577,18 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
 
             } # conditional.x
 
-
+            # group.w.free (new in 0.6-8)
+            if(group.w.free) {
+                lhs    <- c(lhs, "group")
+                 op    <- c(op,  "%")
+                rhs    <- c(rhs, "w")
+                block  <- c(block, b)
+                group  <- c(group, g)
+                level  <- c(level, l)
+                free   <- c(free,  1L)
+                exo    <- c(exo,   0L)
+                ustart <- c(ustart, lavsamplestats@WLS.obs[[g]][1])
+            }
 
         } # levels
     } # ngroups
@@ -589,3 +622,204 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
     LIST
 
 }
+
+# - currently only used for continuous twolevel data
+# - conditional.x not supported (yet)
+lav_partable_unrestricted_chol <- function(lavobject = NULL,
+                                      # if no object is available,
+                                      lavdata        = NULL,
+                                      lavpta         = NULL,
+                                      lavoptions     = NULL) {
+
+    # grab everything from lavaan lavobject
+    if(!is.null(lavobject)) {
+        stopifnot(inherits(lavobject, "lavaan"))
+
+        lavdata <- lavobject@Data
+        lavoptions <- lavobject@Options
+        #lavsamplestats <- lavobject@SampleStats
+        lavpta <- lavobject@pta
+        #lavh1 <- lavobject@h1
+    }
+
+    ov            <- lavdata@ov
+    meanstructure <- lavoptions$meanstructure
+    categorical   <- any(ov$type == "ordered")
+    if(categorical) {
+        stop("lavaan ERROR: categorical data not supported in this function")
+    }
+    ngroups       <- lavdata@ngroups
+    nlevels       <- lavdata@nlevels
+
+    # what with fixed.x?
+    # - does not really matter; fit will be saturated anyway
+    # - fixed.x = TRUE may avoid convergence issues with non-numeric
+    #             x-covariates
+    fixed.x = lavoptions$fixed.x
+
+    # if multilevel
+    if(nlevels > 1L) {
+        #fixed.x       <- FALSE # for now
+        conditional.x <- FALSE # for now
+        categorical   <- FALSE # for now
+    }
+
+    lhs <- rhs <- op <- character(0)
+    group <- block <- level <- free <- exo <- integer(0)
+    ustart <- lower <- numeric(0)
+
+    # block number
+    b <- 0L
+    for(g in 1:ngroups) {
+
+        # only for multilevel
+        if(nlevels > 1L) {
+            Lp <- lavdata@Lp[[g]]
+        }
+
+        for(l in 1:nlevels) {
+
+            # block
+            b <- b + 1L
+
+            ov.names     <- lavpta$vnames$ov[[b]]
+            ov.names.x   <- lavpta$vnames$ov.x[[b]]
+            ov.names.nox <- lavpta$vnames$ov.nox[[b]]
+
+            # only for multilevel, overwrite sample.cov and sample.mean
+            if(nlevels > 1L) {
+                ov.names.x <- character(0L)
+                ov.names.nox <- ov.names
+            }
+
+            # create lv.names == ov.names
+            lv.names <- paste("f", ov.names, sep = "")
+
+            # a) OV VARIANCES -> fixed to zero
+            nvar  <- length(ov.names)
+            lhs   <- c(lhs, ov.names)
+             op   <- c(op, rep("~~", nvar))
+            rhs   <- c(rhs, ov.names)
+            block <- c(block, rep(b,   nvar))
+            group <- c(group, rep(g,   nvar))
+            level <- c(level, rep(l,   nvar))
+            ustart<- c(ustart,rep(0.0001, nvar)) ### Force PD!! (option?)
+            free  <- c(free,  rep(0L,  nvar))
+            exo   <- c(exo,   rep(0L,  nvar))
+            lower <- c(lower, rep(0.0, nvar))
+
+            # b) LV VARIANCES -> fixed to 1.0
+            nvar  <- length(lv.names)
+            lhs   <- c(lhs, lv.names)
+             op   <- c(op, rep("~~", nvar))
+            rhs   <- c(rhs, lv.names)
+            block <- c(block, rep(b,   nvar))
+            group <- c(group, rep(g,   nvar))
+            level <- c(level, rep(l,   nvar))
+            ustart<- c(ustart,rep(1.0, nvar))
+            free  <- c(free,  rep(0L,  nvar))
+            exo   <- c(exo,   rep(0L,  nvar))
+            lower <- c(lower, rep(1.0, nvar))
+
+            # c) LOADINGS self
+            nvar  <- length(ov.names)
+            lhs   <- c(lhs, lv.names)
+             op   <- c(op, rep("=~", nvar))
+            rhs   <- c(rhs, ov.names)
+            block <- c(block, rep(b,   nvar))
+            group <- c(group, rep(g,   nvar))
+            level <- c(level, rep(l,   nvar))
+            ustart<- c(ustart,rep(as.numeric(NA), nvar))
+            free  <- c(free,  rep(1L,  nvar))
+            exo   <- c(exo,   rep(0L,  nvar))
+            lower <- c(lower, rep(0.0, nvar)) # lower bound!
+
+            # d) LOADINGS other
+            if(length(ov.names) > 1L) {
+                tmp <- utils::combn(ov.names, 2)
+                pstar <- ncol(tmp)
+                lhs <- c(lhs, paste("f", tmp[1,], sep = ""))
+                 op <- c(op,   rep("=~", pstar))
+                rhs <- c(rhs, tmp[2,])
+                block <- c(block, rep(b,   pstar))
+                group <- c(group, rep(g,   pstar))
+                level <- c(level, rep(l,   pstar))
+                free  <- c(free,  rep(1L,  pstar))
+                exo   <- c(exo,   rep(0L,  pstar))
+                lower <- c(lower, rep(-Inf, pstar))
+                ustart<- c(ustart,rep(as.numeric(NA), pstar))
+            }
+
+            # meanstructure?
+            if(meanstructure) {
+
+                # OV
+                ov.int <- ov.names
+
+                nel <- length(ov.int)
+                lhs   <- c(lhs, ov.int)
+                 op   <- c(op, rep("~1", nel))
+                rhs   <- c(rhs, rep("", nel))
+                block <- c(block, rep(b,  nel))
+                group <- c(group, rep(g,  nel))
+                level <- c(level, rep(l,  nel))
+                # if multilevel, level=1 has fixed zeroes
+                if(nlevels > 1L && l == 1L) {
+                    WITHIN <- rep(0L, nel)
+                    WITHIN[Lp$within.idx[[2]]] <- 1L
+                    free  <- c(free, WITHIN)
+                } else {
+                    free  <- c(free,  rep(1L, nel))
+                }
+                exo   <- c(exo,   rep(0L, nel))
+                lower <- c(lower, rep(-Inf, nel))
+                ustart <- c(ustart, rep(as.numeric(NA), nel))
+
+                # LV
+                ov.int <- lv.names
+
+                nel    <- length(ov.int)
+                lhs    <- c(lhs,    ov.int)
+                 op    <- c(op,     rep("~1", nel))
+                rhs    <- c(rhs,    rep("",   nel))
+                block  <- c(block,  rep(b,    nel))
+                group  <- c(group,  rep(g,    nel))
+                level  <- c(level,  rep(l,    nel))
+                free   <- c(free,   rep(0L,   nel))
+                exo    <- c(exo,    rep(0L,   nel))
+                ustart <- c(ustart, rep(0.0,  nel))
+                lower  <- c(lower,  rep(-Inf, nel))
+            }
+
+        } # levels
+    } # ngroups
+
+    # free counter
+    idx.free <- which(free > 0)
+    free[idx.free] <- 1:length(idx.free)
+
+    LIST   <- list(     id          = 1:length(lhs),
+                        lhs         = lhs,
+                        op          = op,
+                        rhs         = rhs,
+                        user        = rep(1L,  length(lhs)),
+                        block       = block,
+                        group       = group,
+                        level       = level,
+                        free        = free,
+                        ustart      = ustart,
+                        exo         = exo,
+                        lower       = lower #,
+                        #label       = rep("",  length(lhs))
+                        #eq.id       = rep(0L,  length(lhs)),
+                        #unco        = free
+                   )
+
+
+    # keep level column if no levels? (no for now)
+    if(nlevels < 2L) {
+        LIST$level <- NULL
+    }
+
+    LIST
+}
