@@ -65,16 +65,20 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
                    "lv.nonnormal",# latent variables with non-normal indicators
                    "lv.interaction", # interaction terms
                    "lv.efa",      # latent variables involved in efa
+                   "lv.rv",       # random slopes, random variables
                    "lv.ind",      # latent indicators (higher-order cfa)
+                   "lv.marker",   # marker indicator per lv
 
                    "eqs.y",       # y's in regression
                    "eqs.x"        # x's in regression
                   )
 
     # sanity check
-    stopifnot(is.list(partable),
-              !missing(type),
-              type %in% c(type.list, "all"))
+    stopifnot(is.list(partable), !missing(type))
+
+    if(!type %in% c(type.list, "all")) {
+        stop("lavaan ERROR: type =", dQuote(type), " is not a valid option")
+    }
 
     if(length(type) == 1L && type == "all") {
         type <- type.list
@@ -135,6 +139,13 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
         }
     }
 
+    # random slope names, if any (new in 0.6-7)
+    if(!is.null(partable$rv) && any(nchar(partable$rv) > 0L)) {
+        RV.names <- unique(partable$rv[nchar(partable$rv) > 0L])
+    } else {
+        RV.names <- character(0L)
+    }
+
 
 
     # output: list per block
@@ -163,7 +174,9 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
     OUT$lv.nonnormal   <- vector("list", length = nblocks)
     OUT$lv.interaction <- vector("list", length = nblocks)
     OUT$lv.efa         <- vector("list", length = nblocks)
+    OUT$lv.rv          <- vector("list", length = nblocks)
     OUT$lv.ind         <- vector("list", length = nblocks)
+    OUT$lv.marker      <- vector("list", length = nblocks)
 
     OUT$eqs.y          <- vector("list", length = nblocks)
     OUT$eqs.x          <- vector("list", length = nblocks)
@@ -174,6 +187,8 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
         lv.names <- unique( partable$lhs[ partable$block == b  &
                                           (partable$op == "=~" |
                                            partable$op == "<~")  ] )
+        # including random slope names
+        lv.names2 <- unique(c(lv.names, RV.names))
 
         # determine lv interactions
         int.names <- unique(partable$rhs[ partable$block == b  &
@@ -196,19 +211,28 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
             }
             lv.interaction <- int.names[ok.idx]
             lv.names <- c(lv.names, lv.interaction)
+            lv.names2 <- c(lv.names2, lv.interaction)
         } else {
             lv.interaction <- character(0L)
         }
 
         # store lv
         if("lv" %in% type) {
-            OUT$lv[[b]] <- lv.names
+            # check if FLAT for random slopes
+            #if( !is.null(partable$rv) && any(nchar(partable$rv) > 0L) &&
+            #    !is.null(partable$block) ) {
+            #    OUT$lv[[b]] <- lv.names2
+            #} else {
+                # here, they will be 'defined' at level 2 as regular =~ lvs
+                OUT$lv[[b]] <- lv.names
+            #}
         }
 
         # regular latent variables ONLY (ie defined by =~ only)
         if("lv.regular" %in% type) {
             out <- unique( partable$lhs[ partable$block == b &
-                                         partable$op == "=~"   ] )
+                                         partable$op == "=~" &
+                                         !partable$lhs %in% RV.names ] )
             OUT$lv.regular[[b]] <- out
         }
 
@@ -238,6 +262,18 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
             OUT$lv.efa[[b]] <- out
         }
 
+        # lv's that are random slopes
+        if("lv.rv" %in% type) {
+            if(is.null(partable$rv)) {
+                out <- character(0L)
+            } else {
+                out <- unique( partable$lhs[ partable$op == "=~" &
+                                             partable$block == b &
+                                             partable$lhs %in% RV.names ] )
+            }
+            OUT$lv.rv[[b]] <- out
+        }
+
         # lv's that are indicators of a higher-order factor
         if("lv.ind" %in% type) {
             out <- unique( partable$rhs[ partable$block == b &
@@ -282,11 +318,18 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
         if(!(length(type) == 1L &&
              type %in% c("lv", "lv.regular", "lv.nonnormal", "lv.x","lv.y"))) {
             # 1. indicators, which are not latent variables themselves
-            ov.ind <- v.ind[ !v.ind %in% lv.names ]
+            ov.ind <- v.ind[ !v.ind %in% lv.names2 ]
             # 2. dependent ov's
-            ov.y <- eqs.y[ !eqs.y %in% c(lv.names, ov.ind) ]
+            ov.y <- eqs.y[ !eqs.y %in% c(lv.names2, ov.ind) ]
             # 3. independent ov's
-            ov.x <- eqs.x[ !eqs.x %in% c(lv.names, ov.ind, ov.y) ]
+            if(lav_partable_nlevels(partable) > 1L && b > 1L) {
+                # NEW in 0.6-8: if an 'x' was an 'y' in a previous level,
+                #               treat it as 'y'
+                EQS.Y <- unique(partable$lhs[partable$op == "~"]) # all blocks
+                ov.x <- eqs.x[ !eqs.x %in% c(lv.names2, ov.ind, EQS.Y) ]
+            } else {
+                ov.x <- eqs.x[ !eqs.x %in% c(lv.names2, ov.ind, ov.y) ]
+            }
         }
 
         # observed variables
@@ -299,15 +342,15 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
             # 4. orphaned covariances
             ov.cov <- c(partable$lhs[ partable$block == b &
                                       partable$op == "~~" &
-                                     !partable$lhs %in% lv.names ],
+                                     !partable$lhs %in% lv.names2 ],
                         partable$rhs[ partable$block == b &
                                       partable$op == "~~" &
-                                     !partable$rhs %in% lv.names ])
+                                     !partable$rhs %in% lv.names2 ])
             # 5. orphaned intercepts/thresholds
             ov.int <- partable$lhs[ partable$block == b &
                                     (partable$op == "~1" |
                                      partable$op == "|") &
-                                    !partable$lhs %in% lv.names ]
+                                    !partable$lhs %in% lv.names2 ]
 
             ov.tmp <- c(ov.ind, ov.y, ov.x)
             ov.extra <- unique(c(ov.cov, ov.int)) # must be in this order!
@@ -387,9 +430,17 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
                             "];\n  Please remove them and try again.")
                 }
                 if(warn) {
-                    warning("lavaan WARNING: model syntax contains variance/covariance/intercept formulas\n  involving (an) exogenous variable(s): [",
-                            paste(ov.x[idx.no.x], collapse=" "),
-                            "];\n  Please use fixed.x=FALSE or leave them alone")
+                    txt <- c("model syntax contains ",
+                    "variance/covariance/intercept formulas involving",
+                    " (an) exogenous variable(s): [",
+                    paste(ov.x[idx.no.x], collapse=" "), "]; ",
+                    "These variables will now be treated as random ",
+                    "introducing additional free parameters. ",
+                    "If you wish to treat ",
+                    "those variables as fixed, remove these ",
+                    "formulas from the model syntax. Otherwise, consider ",
+                    "adding the fixed.x = FALSE option.")
+                    warning(lav_txt2message(txt))
                 }
                 ov.x <- ov.x[-idx.no.x]
             }
@@ -573,14 +624,42 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
             OUT$lv.nox[[b]] <- lv.names[! lv.names %in% lv.names.x ]
         }
 
-    }
+        # marker indicator (if any) for each lv
+        if("lv.marker" %in% type) {
+            # default: "" per lv
+            out <- character( length(lv.names) )
+            names(out) <- lv.names
+            for(l in seq_len( length(lv.names) )) {
+                this.lv.name <- lv.names[l]
+                # try to see if we can find a 'marker' indicator for this factor
+                marker.idx <- which(partable$block == b &
+                                    partable$lhs == this.lv.name &
+                                    partable$rhs %in% v.ind &
+                                    partable$ustart == 1L &
+                                    partable$free == 0L)
+                if(length(marker.idx) == 1L) { # unique only!!
+                    out[l] <- partable$rhs[marker.idx]
+                }
+            }
+            OUT$lv.marker[[b]] <- out
+        }
+
+    } # b
 
     # to mimic old behaviour, if length(type) == 1L
     if(length(type) == 1L) {
         OUT <- OUT[[type]]
         # to mimic old behaviour, if specific block is requested
         if(ndotdotdot == 0L) {
-            OUT <- unique(unlist(OUT))
+            if(type == "lv.marker") {
+                OUT <- unlist(OUT)
+                # no unique, as unique drops attributes, and reduces
+                # c("", "", "") to a single ""
+                # (but, say for 2 groups, you get 2 copies)
+                # as this is only for 'display', we leave like that
+            } else {
+                OUT <- unique(unlist(OUT))
+            }
         } else if(length(block.select) == 1L) {
             OUT <- OUT[[block.select]]
         } else {
