@@ -87,6 +87,7 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
 
     # format for numeric values
     num.format  <- paste("%", max(8L, nd + 5L), ".", nd, "f", sep = "")
+    int.format  <- paste("%", max(8L, nd + 5L), "d", sep = "")
     char.format <- paste("%", max(8L, nd + 5L), "s", sep = "")
 
     # output sections
@@ -121,34 +122,72 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
             # container
             c1 <- c2 <- character(0L)
 
+            # which type of standard errors?
+            c1 <- c(c1, "Standard errors")
+            if(attr(x, "se") == "robust.huber.white") {
+                tmp.txt <- "sandwich" # since 0.6-6
+            } else {
+                tmp.txt <- attr(x, "se")
+            }
+            c2 <- c(c2, paste(toupper(substring(tmp.txt, 1, 1)),
+                                      substring(tmp.txt, 2), sep = ""))
+            # information
             if(attr(x, "se") != "bootstrap") {
-                # 1.
-                c1 <- c(c1, "Information")
+
+                # type for information
+                if(attr(x, "se") == "robust.huber.white") {
+                    c1 <- c(c1, "Information bread")
+                } else {
+                    c1 <- c(c1, "Information")
+                }
                 tmp.txt <- attr(x, "information")
                 c2 <- c(c2, paste(toupper(substring(tmp.txt, 1, 1)),
                                   substring(tmp.txt, 2), sep = ""))
 
-                # 2.
-                if(attr(x, "information") %in% c("expected", "first.order") ||
-                   attr(x, "observed.information") == "h1") {
-                    c1 <- c(c1, "Information saturated (h1) model")
-                    tmp.txt <- attr(x, "h1.information")
-                    c2 <- c(c2, paste(toupper(substring(tmp.txt,1,1)),
-                                      substring(tmp.txt, 2), sep = ""))
-                }
+                # if observed, which type? (hessian of h1)
                 if(attr(x, "information") == "observed") {
                     c1 <- c(c1, "Observed information based on")
                     tmp.txt <- attr(x, "observed.information")
                     c2 <- c(c2, paste(toupper(substring(tmp.txt, 1, 1)),
                                       substring(tmp.txt, 2), sep = ""))
                 }
-            } # no bootstrap
 
-            # 3.
-            c1 <- c(c1, "Standard errors")
-            tmp.txt <- attr(x, "se")
-            c2 <- c(c2, paste(toupper(substring(tmp.txt, 1, 1)),
+                # if h1 is involved, structured or unstructured?
+                if(attr(x, "information") %in% c("expected", "first.order") ||
+                   attr(x, "observed.information") == "h1") {
+                    if(attr(x, "se") == "robust.huber.white" &&
+                       attr(x, "h1.information") !=
+                       attr(x, "h1.information.meat")) {
+                        c1 <- c(c1, "Information bread saturated (h1) model")
+                    } else {
+                        c1 <- c(c1, "Information saturated (h1) model")
+                    }
+                    tmp.txt <- attr(x, "h1.information")
+                    c2 <- c(c2, paste(toupper(substring(tmp.txt,1,1)),
                                       substring(tmp.txt, 2), sep = ""))
+                }
+
+                # if sandwich, which information for the meat? (first.order)
+                # only print if it is NOT first.order
+                if(attr(x, "se") == "robust.huber.white" &&
+                   attr(x, "information.meat") != "first.order") {
+                    c1 <- c(c1, "Information meat")
+                    tmp.txt <- attr(x, "information.meat")
+                    c2 <- c(c2, paste(toupper(substring(tmp.txt,1,1)),
+                                      substring(tmp.txt, 2), sep = ""))
+                }
+
+                # if sandwich, structured or unstructured for the meat?
+                # only print if it is not the same as h1.information
+                if(attr(x, "se") == "robust.huber.white" &&
+                   attr(x, "h1.information.meat") !=
+                   attr(x, "h1.information")) {
+                    c1 <- c(c1, "Information meat saturated (h1) model")
+                    tmp.txt <- attr(x, "h1.information.meat")
+                    c2 <- c(c2, paste(toupper(substring(tmp.txt,1,1)),
+                                      substring(tmp.txt, 2), sep = ""))
+                }
+            } # no bootstrap
 
             # 4.
             if(attr(x, "se") == "bootstrap" && !is.null(attr(x, "bootstrap"))) {
@@ -194,10 +233,29 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
         x$block <- rep(1L, length(x$lhs))
     }
 
+    # step column (SAM)
+    #if(!is.null(x$step)) {
+    #    tmp.LABEL <- rep("", length(x$lhs))
+    #    p1.idx <- which(x$step == 1L)
+    #    p2.idx <- which(x$step == 2L)
+    #    tmp.LABEL[p1.idx] <- "1"
+    #    tmp.LABEL[p2.idx] <- "2"
+    #
+    #    if(is.null(x$label)) {
+    #        x$label <- tmp.LABEL
+    #    } else {
+    #        x$label <- paste(x$label, tmp.LABEL, sep = "")
+    #    }
+    #
+    #    x$step <- NULL
+    #}
+
     # round to 3 digits after the decimal point
     y <- as.data.frame(
            lapply(x, function(x) {
-               if(is.numeric(x)) {
+               if(is.integer(x)) {
+                   sprintf(int.format, x)
+               } else if(is.numeric(x)) {
                    sprintf(num.format, x)
                } else {
                    x
@@ -262,6 +320,39 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
         }
     }
 
+    # handle lower/upper boundary points
+    if(!is.null(x$lower)) {
+        b.idx <- which( abs(x$lower - x$est) < sqrt(.Machine$double.eps) &
+                        (is.na(x$se) | (is.finite(x$se) & x$se != 0.0)) )
+        if(length(b.idx) > 0L && !is.null(x$pvalue)) {
+            m[b.idx, "pvalue"] <- ""
+            if(is.null(x$label)) {
+                x$label <- rep("", length(x$lhs))
+            }
+            x$label[b.idx] <- ifelse(nchar(x$label[b.idx]) > 0L,
+                                     paste(x$label[b.idx], "+lb", sep = ""),
+                                     "lb")
+        }
+        # remove lower column
+        m <- m[, colnames(m) != "lower"]
+    }
+    if(!is.null(x$upper)) {
+        b.idx <- which( abs(x$upper - x$est) < sqrt(.Machine$double.eps) &
+                        is.finite(x$se) & x$se != 0.0)
+        if(length(b.idx) > 0L && !is.null(x$pvalue)) {
+            m[b.idx, "pvalue"] <- ""
+            if(is.null(x$label)) {
+                x$label <- rep("", length(x$lhs))
+            }
+            x$label[b.idx] <- ifelse(nchar(x$label[b.idx]) > 0L,
+                                     paste(x$label[b.idx], "+ub", sep = ""),
+                                     "ub")
+        }
+        # remove upper column
+        m <- m[, colnames(m) != "upper"]
+    }
+
+
     # handle fmi
     if(!is.null(x$fmi)) {
         se.idx <- which(x$se == 0)
@@ -310,6 +401,7 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
     colnames(m)[ colnames(m) ==    "lhs" ] <- ""
     colnames(m)[ colnames(m) ==     "op" ] <- ""
     colnames(m)[ colnames(m) ==    "rhs" ] <- ""
+    colnames(m)[ colnames(m) ==   "step" ] <- "Step"
     colnames(m)[ colnames(m) ==    "est" ] <- "Estimate"
     colnames(m)[ colnames(m) ==     "se" ] <- "Std.Err"
     colnames(m)[ colnames(m) ==      "z" ] <- "z-value"
@@ -688,7 +780,7 @@ print.lavaan.fsr <- function(x, ..., nd = 3L, mm = FALSE, struc = FALSE) {
         lav_object_print_short_summary(y$STRUC.FIT)
         FIT <- fitMeasures(y$STRUC.FIT, fit.measures="default")
         if(FIT["df"] > 0) {
-            print.lavaan.fitMeasures( FIT )
+            print.lavaan.fitMeasures( FIT, add.h0 = FALSE )
         }
     }
     PE <- parameterEstimates(y$STRUC.FIT, ci = FALSE,
