@@ -296,7 +296,7 @@ lavParseModelString <- function(model.syntax = '', as.data.frame. = FALSE,
         RHS <- strsplit(rhs, split = "+", fixed = TRUE)[[1]]
         RHS.names <- gsub("^\\S*\\*", "", RHS)
         BAD <- c("if", "else", "repeat", "while", "function", "for", "in")
-        if(any(RHS.names %in% BAD)) {
+        if(any(RHS.names %in% c(BAD, "NA"))) { # "NA" added in 0.6-8
             stop("lavaan ERROR: right hand side (rhs) of this formula:\n    ",
                  lhs, " ", op, " ", rhs,
                  "\n    contains either a reserved word (in R) or an illegal character: ",
@@ -382,6 +382,17 @@ lavParseModelString <- function(model.syntax = '', as.data.frame. = FALSE,
                         stop("lavaan ERROR: duplicate model element in: ", model[i])
                     }
                 }
+
+                # check if we have a self-loop (y ~ y)
+                if(op %in% c("~", "<~") && rhs.name == lhs.names[l]) {
+                    #stop("lavaan ERROR: lhs and rhs are the same in: ", 
+                    #     model[i])
+                    # this breaks pompom package, example uSEM
+                    warning("lavaan WARNING: lhs and rhs are the same in: ", 
+                            model[i])
+                }
+                 
+
                 FLAT.idx <- FLAT.idx + 1L
                 FLAT.lhs[FLAT.idx] <- lhs.names[l]
                 FLAT.op[ FLAT.idx] <- op
@@ -396,7 +407,6 @@ lavParseModelString <- function(model.syntax = '', as.data.frame. = FALSE,
                 FLAT.efa[FLAT.idx]   <- ""
                 FLAT.rv[FLAT.idx]    <- ""
 
-
                 mod <- list()
                 rhs.mod <- 0L
                 if(length(lhs.out[[l]]$efa) > 0L) {
